;;;;
;;;; anaphoric -- Anaphoric macros like those found in Paul Graham's On Lisp book.
;;;;
;;;; Copyright 2005 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-general.
;;;;
;;;; octw-guile-general is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-general is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-general.  If not, see
;;;; <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: anaphoric.scm,v 1.2 2008-09-08 18:54:52 dean Exp $

(define-module (octw anaphoric))

(export aif
        awhen
        aand
        acond rewrite-acond-clause acond-aux)

(use-syntax (ice-9 syncase))

(define-syntax aif
  (lambda (form)
    (syntax-case form ()
      ((_ test consequent alternate ...)
       (with-syntax ((it (datum->syntax-object (syntax test) 'it)))
         (syntax
          (let ((it test))
            (if it consequent alternate ...))))))))

(define-syntax awhen
  (lambda (form)
    (syntax-case form ()
      ((_ test body ...)
       (syntax (aif test (begin body ...)))))))

(define-syntax aand
  (lambda (form)
    (syntax-case form ()
      ((_)
       (syntax #t))
      ((_ expr1)
       (syntax expr1))
      ((_ expr1 expr2)
       (syntax (aif expr1 expr2 #f)))
      ((_ expr1 expr2 expr ...)
       (syntax (aif expr1
                    (aand expr2 expr ...)
                    #f))))))

(define (rewrite-acond-clause clause)
  (syntax-case clause (else =>)
    ;; Handle the => case
    ((test => expression)
     (syntax
      (test => expression)))
    ;; Handle the else case
    ((else body1 bodyN ...)
     (syntax
      (else body1 bodyN ...)))
    ;; Handle the no expression case
    ((test)
     (syntax
      (test)))
    ;; Handle the case where "it" needs binding
    ((test body1 bodyN ...)
     (with-syntax ((it (datum->syntax-object (syntax test) 'it)))
       (syntax
        (test => (lambda (it)
                   body1 bodyN ...)))))))

(define-syntax acond-aux
  (lambda (form)
    (syntax-case form ()
      ((_ (clause1 clauseN ...) converted ...)
       (with-syntax ((clause1 (rewrite-acond-clause (syntax clause1))))
         (syntax
          (acond-aux (clauseN ...) converted ... clause1))))
      ((_ () converted ...)
       (syntax
        (cond converted ...))))))

(define-syntax acond
  (lambda (form)
    (syntax-case form ()
      ((_ clause1 clauseN ...)
       (syntax
        (acond-aux (clause1 clauseN ...)))))))
